@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_audio.c,v 1.7 2001/06/16 01:51:42 hercules Exp $";
+ "@(#) $Id: SDL_audio.c,v 1.9 2001/07/23 02:58:42 slouken Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -40,6 +40,9 @@ static char rcsid =
 
 /* Available audio drivers */
 static AudioBootStrap *bootstrap[] = {
+#ifdef OPENBSD_AUDIO_SUPPORT
+	&OPENBSD_AUDIO_bootstrap,
+#endif
 #ifdef OSS_SUPPORT
 	&DSP_bootstrap,
 	&DMA_bootstrap,
@@ -47,9 +50,6 @@ static AudioBootStrap *bootstrap[] = {
 #ifdef ALSA_SUPPORT
 	&ALSA_bootstrap,
 #endif
-#ifdef __OpenBSD__
-	&OBSD_bootstrap,
-#endif
 #if (defined(unix) && !defined(__CYGWIN32__)) && \
     !defined(OSS_SUPPORT) && !defined(ALSA_SUPPORT)
 	&AUDIO_bootstrap,
