@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dspaudio.c,v 1.3 2001/05/23 00:36:17 hercules Exp $";
+ "@(#) $Id: SDL_dspaudio.c,v 1.7 2001/07/09 15:47:40 hercules Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -37,7 +37,13 @@ static char rcsid =
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#ifdef OSS_USE_SOUNDCARD_H
+/* This is installed on some systems */
+#include <soundcard.h>
+#else
+/* This is recommended by OSS */
 #include <sys/soundcard.h>
+#endif
 
 #include "SDL_audio.h"
 #include "SDL_error.h"
@@ -52,11 +58,7 @@ static char rcsid =
 
 /* Open the audio device for playback, and don't block if busy */
 /*#define USE_BLOCKING_WRITES*/
-#ifdef USE_BLOCKING_WRITES
-#define OPEN_FLAGS	O_WRONLY
-#else
 #define OPEN_FLAGS	(O_WRONLY|O_NONBLOCK)
-#endif
 
 /* Audio driver functions */
 static int DSP_OpenAudio(_THIS, SDL_AudioSpec *spec);
@@ -128,9 +130,6 @@ AudioBootStrap DSP_bootstrap = {
 /* This function waits until it is possible to write a full sound buffer */
 static void DSP_WaitAudio(_THIS)
 {
-#ifndef USE_BLOCKING_WRITES /* Not necessary because of blocking writes */
-	fd_set fdset;
-
 	/* Check to see if the thread-parent process is still alive */
 	{ static int cnt = 0;
 		/* Note that this only works with thread implementations 
@@ -143,6 +142,7 @@ static void DSP_WaitAudio(_THIS)
 		}
 	}
 
+#ifndef USE_BLOCKING_WRITES /* Not necessary when using blocking writes */
 	/* See if we need to use timed audio synchronization */
 	if ( frame_ticks ) {
 		/* Use timer for general audio synchronization */
@@ -154,7 +154,9 @@ static void DSP_WaitAudio(_THIS)
 		}
 	} else {
 		/* Use select() for audio synchronization */
+		fd_set fdset;
 		struct timeval timeout;
+
 		FD_ZERO(&fdset);
 		FD_SET(audio_fd, &fdset);
 		timeout.tv_sec = 10;
@@ -186,16 +188,24 @@ static void DSP_WaitAudio(_THIS)
 
 static void DSP_PlayAudio(_THIS)
 {
-	int written;
+	int written, p=0;
 
 	/* Write the audio data, checking for EAGAIN on broken audio drivers */
 	do {
-		written = write(audio_fd, mixbuf, mixlen);
-		if ( (written < 0) && ((errno == 0) || (errno == EAGAIN)) ) {
+		written = write(audio_fd, &mixbuf[p], mixlen-p);
+		if (written>0)
+		   p += written;
+		if (written == -1 && errno != 0 && errno != EAGAIN && errno != EINTR)
+		{
+		   /* Non recoverable error has occurred. It should be reported!!! */
+		   perror("audio");
+		   break;
+		}
+
+		if ( p < written || ((written < 0) && ((errno == 0) || (errno == EAGAIN))) ) {
 			SDL_Delay(1);	/* Let a little CPU time go by */
 		}
-	} while ( (written < 0) && 
-	          ((errno == 0) || (errno == EAGAIN) || (errno == EINTR)) );
+	} while ( p < written );
 
 	/* If timer synchronization is enabled, set the next write frame */
 	if ( frame_ticks ) {
@@ -325,6 +335,18 @@ static int DSP_OpenAudio(_THIS, SDL_AudioSpec *spec)
 	}
 	mixbuf = NULL;
 
+#ifdef USE_BLOCKING_WRITES
+	/* Make the file descriptor use blocking writes with fcntl() */
+	{ long flags;
+		flags = fcntl(audio_fd, F_GETFL);
+		flags &= ~O_NONBLOCK;
+		if ( fcntl(audio_fd, F_SETFL, flags) < 0 ) {
+			SDL_SetError("Couldn't set audio blocking mode");
+			return(-1);
+		}
+	}
+#endif
+
 	/* Get a list of supported hardware formats */
 	if ( ioctl(audio_fd, SNDCTL_DSP_GETFMTS, &value) < 0 ) {
 		SDL_SetError("Couldn't get audio format list");
