@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_syssem.c,v 1.3 2001/05/10 18:31:21 hercules Exp $";
+ "@(#) $Id: SDL_syssem.c,v 1.4 2001/07/08 09:00:06 hercules Exp $";
 #endif
 
 #include <stdlib.h>
@@ -48,7 +48,7 @@ static char rcsid =
 #ifdef SDL_USE_PTHREADS
 
 #ifdef SDL_NO_PTHREAD_SEMAPHORES
-#include "generic/SDL_sem.c"
+#include "generic/SDL_syssem.c"
 #else
 
 #include <stdio.h>
