@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysvideo.h,v 1.5 2001/06/07 14:28:11 hercules Exp $";
+ "@(#) $Id: SDL_sysvideo.h,v 1.6 2001/06/19 13:33:53 hercules Exp $";
 #endif
 
 #ifndef _SDL_sysvideo_h
@@ -343,6 +343,9 @@ extern VideoBootStrap PS2GS_bootstrap;
 #ifdef ENABLE_GGI
 extern VideoBootStrap GGI_bootstrap;
 #endif
+#ifdef ENABLE_VGL
+extern VideoBootStrap VGL_bootstrap;
+#endif
 #ifdef ENABLE_SVGALIB
 extern VideoBootStrap SVGALIB_bootstrap;
 #endif
