@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_video.c,v 1.10 2001/06/11 06:44:43 hercules Exp $";
+ "@(#) $Id: SDL_video.c,v 1.11 2001/06/19 13:33:53 hercules Exp $";
 #endif
 
 /* The high-level video driver subsystem */
@@ -63,6 +63,9 @@ static VideoBootStrap *bootstrap[] = {
 #ifdef ENABLE_GGI
 	&GGI_bootstrap,
 #endif
+#ifdef ENABLE_VGL
+	&VGL_bootstrap,
+#endif
 #ifdef ENABLE_SVGALIB
 	&SVGALIB_bootstrap,
 #endif
